# Copyright 2016 You Know WHO.  It's GPL.

import sys
import random
import splat.data
import splat.filters
import splat.gen
import splat.seq
import splat.interpol
import splat.tools.make
import settings
from splat import dB

# This layer is entirely made of drops of water samples.

SPLAT_DEPS = ['settings.py']

def _set(name, variants):
    return splat.seq.SampleSet(list(
        (splat.data.Fragment.open('samples/{}{}.wav'.format(name, x)), 1, 1.0)
        for x in variants))

class Bar(splat.seq.Pattern):
    drops = splat.seq.SampleSet(list(
            (splat.data.Fragment.open(
                    'samples/water-002-drop{:02d}.wav'.format(i)), 1, 1.0)
             for i in range(1, 33)))
    wood = _set('wood-002', 'abcdefghijkl')
    ding = _set('taps-001', 'ab')

    def __init__(self, *args, **kw):
        super(Bar, self).__init__(*args, **kw)


class BarDrops(Bar):
    def play(self, frag, bar, beat, t, T):
        m = 1
        n = 1
        for i in range(12):
            i = m + n
            m = n
            n = i
            pan = random.random() * 4.0
            self.pick_mix(frag, self.drops, t+T*4*m/n,
                          levels=(dB(pan-n), dB(4.0-pan-n)))


class BarWoodA(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat % 2 == 0:
            self.pick_mix(frag, self.wood, t)


class BarWoodB(Bar):
    def play(self, frag, bar, beat, t, T):
        n = 6
        for i in range(6):
            lvl_l = random.uniform(0, 3)
            lvl_r = 3 - lvl_l
            self.pick_mix(frag, self.wood, t + T * i / 6,
                          levels=(dB(lvl_l-beat*3), dB(lvl_r-beat*3)))


class BarEnd(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat == 3:
            for i in range(16):
                pan = random.random() * 2.0
                self.pick_mix(frag, self.ding, t+T*(16-i)/8,
                              levels=(dB(pan-i), dB(2.0-pan-i)))
            self.pick_mix(frag, self.ding, t+T*4.0)
            

def hiss(frag):
    hiss = splat.data.Fragment.open('samples/lampshade-005y.wav')
    splat.filters.linear_fade(hiss, 3.8)
    hiss1 = hiss.dup()
    hiss2 = hiss.dup()

    envelope = splat.interpol.spline([
            (0.0, 0.0, 0.0),
            (2.5, 0.0, 0.0),
            (hiss.duration / 2, dB(-24.0)),
            (hiss.duration - 1.5, 0.0, 0.0),
            (hiss.duration, 0.0, 0.0)]).signal

    resample_ratio = splat.data.Fragment(channels=1, rate=hiss.rate)
    splat.gen.TriangleGenerator(frag=resample_ratio).run(
        0, hiss.duration, freq=3.0*settings.TEMPO/60, levels=envelope)
    resample_ratio.offset(1.0)
    hiss1.resample(ratio=resample_ratio)
    frag.mix(hiss1, levels=(dB(-5), dB(-9)))

    resample_ratio = splat.data.Fragment(channels=1, rate=hiss.rate)
    splat.gen.TriangleGenerator(frag=resample_ratio).run(
        0, hiss.duration, freq=3.001*settings.TEMPO/60, levels=envelope)
    resample_ratio.offset(1.0)
    hiss2.resample(ratio=resample_ratio)
    frag.mix(hiss2, levels=(dB(-9), dB(-5)))

def run(argv):
    frag = splat.data.Fragment()
    Drops = BarDrops()
    WoodA = BarWoodA()
    WoodB = BarWoodB()
    End = BarEnd()
    seq = [(Drops, WoodA)] * 8 + [(Drops, WoodB)] * 7 + [(Drops, WoodB, End)]
    splat.seq.PatternSequencer(settings.TEMPO).run(frag, seq)
    hiss(frag)
    return frag

if __name__ == '__main__':
    splat.tools.make.main_frag(sys.argv, run)
