# Copyright 2016 You Know WHO.  It's GPL.

import sys
import random
import splat.data
import splat.filters
import splat.seq
import splat.tools.make
import settings
from splat import dB

# This layer is all about that bass, no treble.

SPLAT_DEPS = ['settings.py']

def _set(name, variants):
    return splat.seq.SampleSet(list(
        (splat.data.Fragment.open('samples/{}{}.wav'.format(name, x)), 1, 1.0)
        for x in variants))

class Bar(splat.seq.Pattern):
    dong = _set('window-006', 'abcd')
    xip = _set('zip-003', 'abc')
    switch = _set('switch-004', 'ab')
    dung = _set('lampshade-001', 'abcdefg')
    dang = _set('lampshade-003', 'abcdefgh')
    ding = _set('taps-001', 'ab')
    gong = _set('window-001', 'abcde')
    wood = _set('wood-002', 'abcdefghijkl')


class Ang(Bar):
    def play(self, frag, bar, beat, t, T):
        if bar % 2 == 0:
            if beat == 2:
                self.pick_mix(frag, self.dung, t)
            elif beat == 3:
                self.pick_mix(frag, self.dang, t + T/2, dB(0))
        else:
            if beat == 1:
                self.pick_mix(frag, self.dung, t + T/2)
                self.pick_mix(frag, self.dung, t + T*3/4, dB(-6))


class Boom(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat == 0:
            self.pick_mix(frag, self.dong, t)


class Click(Bar):
    def play(self, frag, bar, beat, t, T):
        lvl = -9 if beat % 2 else 0
        self.pick_mix(frag, self.switch, t + T/2, dB(lvl))


class Zeeeeek(Bar):
    def play(self, frag, bar, beat, t, T):
        if (bar % 2 == 1) and beat == 3:
            xip = self.xip.pick().dup()
            if (bar % 4) == 3:
                splat.filters.reverse(xip)
            self.mix(frag, xip, t + T - xip.duration, dB(-8))

class Yeeek(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat == 3:
            xip = self.xip.pick().dup()
            xip2 = xip.dup()
            splat.filters.reverse(xip2)
            xip.mix(xip2)
            self.mix(frag, xip, t, dB(-18))

class Eek(Bar):
    def play(self, frag, bar, beat, t, T):
        self.pick_mix(frag, self.ding, t, dB(12 - (beat * 3)))


class Feek(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat % 2:
            self.pick_mix(frag, self.ding, t, dB(9))
            self.pick_mix(frag, self.ding, t+ T/2, dB(6))


class Gong(Bar):
    def play(self, frag, bar, beat, t, T):
        if beat % 2 == 0:
            lvl = -3 if beat == 0 else -15
            self.pick_mix(frag, self.gong, t, dB(lvl))


class Shower(Bar):
    shower = splat.data.Fragment.open('samples/shower-003a.wav')

    def play(self, frag, bar, beat, t, T):
        if beat == 0:
            frag.mix(self.shower, t - 0.5, dB(-3))

def run(argv):
    frag = splat.data.Fragment()
    A = Ang()
    B = Boom()
    C = Click()
    E = Eek()
    F = Feek()
    G = Gong()
    S = Shower()
    Y = Yeeek()
    Z = Zeeeeek()
    seqA = [B] + [(B, Z)] * 3 + [(B, Z, C)] * 4 + [(A, B, C, Z)] * 6 + [S, E]
    seqB = [(A, B, C), (A, C, E, F), (C, A, B), (G, E)]
    seqC = [(G), (G, C, E), (G, Y), (G, A, C)]
    seq = seqA + seqB * 3 + seqC * 2 + seqB * 3 + seqC * 2
    splat.seq.PatternSequencer(settings.TEMPO).run(frag, seq)
    return frag

if __name__ == '__main__':
    splat.tools.make.main_frag(sys.argv, run)
