# Copyright 2016 You Know WHO.  It's GPL.

import sys
import random
import splat.data
import splat.seq
import splat.tools.make
import settings
from splat import dB
import resampgen

# This layer is the crazy one with the new ResampleGenerator

SPLAT_DEPS = ['settings.py', 'resampgen.py']

VERBOSE = False

class Bar(splat.seq.Pattern):
    brush = splat.data.Fragment.open('samples/zip-003x.wav')
    glass = splat.data.Fragment.open('samples/lampshade-005x.wav')
    glass.amp(dB(24.0))

    def __init__(self, *args, **kw):
        super(Bar, self).__init__(*args, **kw)
        self.gen1 = resampgen.ResampleGenerator({ 1.0: self.brush })
        self.gen2 = resampgen.ResampleGenerator({ 1.0: self.glass })

    def _random_ratio(self, choice=[2, 4, 5, 6, 7]):
        a = float(random.choice(choice))
        b = a + 1.0
        return a / b

    def play(self, frag, bar, beat, t, T):
        if VERBOSE and beat == 0:
            print('bar: {}'.format(bar))
        self.gen1.frag = self.gen2.frag = frag


class Zip1(Bar):
    def play(self, frag, bar, beat, t, T):
        super(Zip1, self).play(frag, bar, beat, t, T)
        if beat == 0:
            self.gen1.run(t+T/4, t+T, 1.0/2.0, levels=(dB(-2), dB(-4)))
        elif beat == 2:
            self.gen1.run(t+T/4, t+T/2, self._random_ratio(),
                          levels=(dB(-4), dB(-2)))
        elif beat == 3:
            self.gen1.run(t, t+T/4, self._random_ratio(), levels=dB(-9))
            self.gen1.run(t+T/2, t+3*T/4, 3.0/4.0, levels=dB(-6))


class Zip2(Bar):
    def play(self, frag, bar, beat, t, T):
        super(Zip2, self).play(frag, bar, beat, t, T)
        if beat % 2:
            self.gen2.run(t+3*T/4, t+T, self._random_ratio(),
                          levels=dB(0-beat))
        else:
            ratio = self._random_ratio()
            self.gen2.run(t, t+T, ratio, levels=(dB(-3-beat), dB(-6-beat)))
            self.gen2.run(t+T/16, t+T, ratio/1.5,
                          levels=(dB(-6-beat), dB(-3-beat)))


class Zip3(Bar):
    def play(self, frag, bar, beat, t, T):
        super(Zip3, self).play(frag, bar, beat, t, T)
        if beat % 2:
            ratio = self._random_ratio()
            self.gen1.run(t+T/2, t+T, ratio, levels=(dB(-3), dB(-6)))
            self.gen1.run(t+T/2+T/16, t+T, ratio*1.5, levels=(dB(-6), dB(-3)))
        else:
            self.gen2.run(t, t+T, 1.0, levels=dB(0))


def run(argv):
    frag = splat.data.Fragment()
    Z1 = Zip1()
    Z2 = Zip2()
    Z3 = Zip3()
    S = splat.seq.Silence()
    seqA = [S] * 8
    seqB = [Z1] * 2
    seqC = [Z2, Z3]
    seq = seqA + seqB * 3 + seqC * 2 + seqB * 3 + seqC * 2
    splat.seq.PatternSequencer(settings.TEMPO).run(frag, seq)
    return frag

if __name__ == '__main__':
    splat.tools.make.main_frag(sys.argv, run)
